@@ -17,7 +17,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file cpu_mmu.c
- * @version 1.0
  * @author Anup Patel (anup@brainfault.org)
  * @brief source file for memory management unit
  */
@@ -25,11 +24,12 @@
 #include <vmm_error.h>
 #include <vmm_string.h>
 #include <vmm_host_aspace.h>
-#include <vmm_cpu.h>
 #include <vmm_main.h>
 #include <vmm_stdio.h>
 #include <vmm_types.h>
+#include <arch_cpu.h>
 #include <cpu_defines.h>
+#include <cpu_barrier.h>
 #include <cpu_inline_asm.h>
 #include <cpu_mmu.h>
 
@@ -41,27 +41,25 @@
 u8 __attribute__((aligned(TTBL_L1TBL_SIZE))) defl1_mem[TTBL_L1TBL_SIZE];
 
 struct cpu_mmu_ctrl {
-	cpu_l1tbl_t defl1;
+	struct cpu_l1tbl defl1;
 	virtual_addr_t l1_base_va;
 	physical_addr_t l1_base_pa;
-	cpu_l1tbl_t * l1_array;
+	struct cpu_l1tbl * l1_array;
 	u8 * l1_bmap;
 	u32 l1_alloc_count;
 	virtual_addr_t l2_base_va;
 	physical_addr_t l2_base_pa;
-	cpu_l2tbl_t * l2_array;
+	struct cpu_l2tbl * l2_array;
 	u8 * l2_bmap;
 	u32 l2_alloc_count;
 	struct dlist l1tbl_list;
 	struct dlist l2tbl_list;
 };
 
-typedef struct cpu_mmu_ctrl cpu_mmu_ctrl_t;
-
-cpu_mmu_ctrl_t mmuctrl;
+static struct cpu_mmu_ctrl mmuctrl;
 
 /** Find L2 page table at given physical address from L1 page table */
-cpu_l2tbl_t *cpu_mmu_l2tbl_find_tbl_pa(physical_addr_t tbl_pa)
+struct cpu_l2tbl *cpu_mmu_l2tbl_find_tbl_pa(physical_addr_t tbl_pa)
 {
 	u32 tmp = (tbl_pa - mmuctrl.l2_base_pa) >> TTBL_L2TBL_SIZE_SHIFT;
 
@@ -73,7 +71,7 @@ cpu_l2tbl_t *cpu_mmu_l2tbl_find_tbl_pa(physical_addr_t tbl_pa)
 }
 
 /** Check whether a L2 page table is attached or not */
-int cpu_mmu_l2tbl_is_attached(cpu_l2tbl_t * l2)
+int cpu_mmu_l2tbl_is_attached(struct cpu_l2tbl * l2)
 {
 	if (!l2) {
 		return 0;
@@ -82,7 +80,7 @@ int cpu_mmu_l2tbl_is_attached(cpu_l2tbl_t * l2)
 }
 
 /** Detach a L2 page table */
-int cpu_mmu_l2tbl_detach(cpu_l2tbl_t * l2)
+int cpu_mmu_l2tbl_detach(struct cpu_l2tbl * l2)
 {
 	u32 *l1_tte;
 	u32 l1_tte_type;
@@ -115,7 +113,7 @@ int cpu_mmu_l2tbl_detach(cpu_l2tbl_t * l2)
 }
 
 /** Attach a L2 page table to a particular L1 page table */
-int cpu_mmu_l2tbl_attach(cpu_l1tbl_t * l1, cpu_l2tbl_t * l2, u32 new_imp,
+int cpu_mmu_l2tbl_attach(struct cpu_l1tbl * l1, struct cpu_l2tbl * l2, u32 new_imp,
 			 u32 new_domain, virtual_addr_t new_map_va, bool force)
 {
 	int rc;
@@ -165,13 +163,13 @@ int cpu_mmu_l2tbl_attach(cpu_l1tbl_t * l1, cpu_l2tbl_t * l2, u32 new_imp,
 }
 
 /** Allocate a L2 page table of given type */
-cpu_l2tbl_t *cpu_mmu_l2tbl_alloc(void)
+struct cpu_l2tbl *cpu_mmu_l2tbl_alloc(void)
 {
 	u32 i;
-	cpu_l2tbl_t *l2;
+	struct cpu_l2tbl *l2;
 
 	if (!list_empty(&mmuctrl.l2tbl_list)) {
-		return list_entry(mmuctrl.l2tbl_list.next, cpu_l2tbl_t, head);
+		return list_entry(mmuctrl.l2tbl_list.next, struct cpu_l2tbl, head);
 	}
 
 	if (mmuctrl.l2_alloc_count < TTBL_MAX_L2TBL_COUNT) {
@@ -202,7 +200,7 @@ cpu_l2tbl_t *cpu_mmu_l2tbl_alloc(void)
 }
 
 /** Free a L2 page table */
-int cpu_mmu_l2tbl_free(cpu_l2tbl_t * l2)
+int cpu_mmu_l2tbl_free(struct cpu_l2tbl * l2)
 {
 	int rc;
 
@@ -217,16 +215,14 @@ int cpu_mmu_l2tbl_free(cpu_l2tbl_t * l2)
 		}
 	}
 
-	list_del(&l2->head);
-
-	mmuctrl.l2_bmap[l2->l2_num] = 0;
+	mmuctrl.l2_bmap[l2->num] = 0;
 	mmuctrl.l2_alloc_count--;
 
 	return VMM_OK;
 }
 
 /** Find L1 page table at given physical address */
-cpu_l1tbl_t *cpu_mmu_l1tbl_find_tbl_pa(physical_addr_t tbl_pa)
+struct cpu_l1tbl *cpu_mmu_l1tbl_find_tbl_pa(physical_addr_t tbl_pa)
 {
 	u32 tmp;
 
@@ -260,14 +256,14 @@ u32 cpu_mmu_best_page_size(virtual_addr_t va, physical_addr_t pa, u32 availsz)
 	return TTBL_L2TBL_SMALL_PAGE_SIZE;
 }
 
-int cpu_mmu_get_page(cpu_l1tbl_t * l1, virtual_addr_t va, cpu_page_t * pg)
+int cpu_mmu_get_page(struct cpu_l1tbl * l1, virtual_addr_t va, struct cpu_page * pg)
 {
 	int ret = VMM_EFAIL;
 	u32 *l1_tte, *l2_tte;
 	u32 l1_tte_type, l1_sec_type;
 	physical_addr_t l2base;
-	cpu_l2tbl_t *l2;
-	cpu_page_t r;
+	struct cpu_l2tbl *l2;
+	struct cpu_page r;
 
 	if (!l1) {
 		return VMM_EFAIL;
@@ -283,7 +279,7 @@ int cpu_mmu_get_page(cpu_l1tbl_t * l1, virtual_addr_t va, cpu_page_t * pg)
 
 	switch (l1_tte_type) {
 	case TTBL_L1TBL_TTE_TYPE_FAULT:
-		vmm_memset(pg, 0, sizeof(cpu_page_t));
+		vmm_memset(pg, 0, sizeof(struct cpu_page));
 		break;
 	case TTBL_L1TBL_TTE_TYPE_SECTION:
 		pg->va = va & TTBL_L1TBL_TTE_OFFSET_MASK;
@@ -373,7 +369,7 @@ int cpu_mmu_get_page(cpu_l1tbl_t * l1, virtual_addr_t va, cpu_page_t * pg)
 		}
 		break;
 	default:
-		vmm_memset(pg, 0, sizeof(cpu_page_t));
+		vmm_memset(pg, 0, sizeof(struct cpu_page));
 		ret = VMM_ENOTAVAIL;
 		break;
 	};
@@ -381,14 +377,14 @@ int cpu_mmu_get_page(cpu_l1tbl_t * l1, virtual_addr_t va, cpu_page_t * pg)
 	return ret;
 }
 
-int cpu_mmu_unmap_page(cpu_l1tbl_t * l1, cpu_page_t * pg)
+int cpu_mmu_unmap_page(struct cpu_l1tbl * l1, struct cpu_page * pg)
 {
 	int ret = VMM_EFAIL;
 	u32 ite, *l1_tte, *l2_tte;
 	u32 l1_tte_type, l1_sec_type, found = 0;
 	physical_addr_t l2base, pgpa, chkpa;
 	virtual_size_t chksz;
-	cpu_l2tbl_t *l2 = NULL;
+	struct cpu_l2tbl *l2 = NULL;
 
 	if (!l1 || !pg) {
 		return ret;
@@ -504,7 +500,7 @@ int cpu_mmu_unmap_page(cpu_l1tbl_t * l1, cpu_page_t * pg)
 	return ret;
 }
 
-int cpu_mmu_map_page(cpu_l1tbl_t * l1, cpu_page_t * pg)
+int cpu_mmu_map_page(struct cpu_l1tbl * l1, struct cpu_page * pg)
 {
 	int rc = VMM_OK;
 	u32 *l1_tte, *l2_tte;
@@ -512,8 +508,8 @@ int cpu_mmu_map_page(cpu_l1tbl_t * l1, cpu_page_t * pg)
 	virtual_addr_t pgva;
 	virtual_size_t pgsz, minpgsz;
 	physical_addr_t l2base;
-	cpu_page_t upg;
-	cpu_l2tbl_t *l2;
+	struct cpu_page upg;
+	struct cpu_l2tbl *l2;
 
 	if (!l1 || !pg) {
 		rc = VMM_EFAIL;
@@ -669,13 +665,13 @@ mmu_map_return:
 	return rc;
 }
 
-static int cpu_mmu_split_reserved_page(cpu_page_t *pg, virtual_size_t rsize)
+static int cpu_mmu_split_reserved_page(struct cpu_page *pg, virtual_size_t rsize)
 {
 	int rc = VMM_EFAIL;
 	int i, count;
 	u32 *l2_tte;
-	cpu_l1tbl_t *l1;
-	cpu_l2tbl_t *l2;
+	struct cpu_l1tbl *l1;
+	struct cpu_l2tbl *l2;
 	virtual_addr_t va;
 	physical_addr_t pa;
 
@@ -759,14 +755,17 @@ error:
 	return rc;
 }
 
-int cpu_mmu_get_reserved_page(virtual_addr_t va, cpu_page_t * pg)
+int cpu_mmu_get_reserved_page(virtual_addr_t va, struct cpu_page * pg)
 {
 	return cpu_mmu_get_page(&mmuctrl.defl1, va, pg);
 }
 
-int cpu_mmu_unmap_reserved_page(cpu_page_t * pg)
+int cpu_mmu_unmap_reserved_page(struct cpu_page * pg)
 {
 	int rc;
+	struct dlist * le;
+	struct cpu_l1tbl * l1;
+	irq_flags_t flags;
 
 	if (!pg) {
 		return VMM_EFAIL;
@@ -776,10 +775,25 @@ int cpu_mmu_unmap_reserved_page(cpu_page_t * pg)
 		return rc;
 	}
 
+	/* Note: It might be possible that the reserved page
+	 * was mapped on-demand into l1 tables other than 
+	 * default l1 table so, we should try to remove mappings
+	 * of this page from other l1 tables.
+	 */
+
+	flags = arch_cpu_irq_save();
+
+	list_for_each(le, &mmuctrl.l1tbl_list) {
+		l1 = list_entry(le, struct cpu_l1tbl, head);
+		cpu_mmu_unmap_page(l1, pg);
+	}
+
+	arch_cpu_irq_restore(flags);
+
 	return VMM_OK;
 }
 
-int cpu_mmu_map_reserved_page(cpu_page_t * pg)
+int cpu_mmu_map_reserved_page(struct cpu_page * pg)
 {
 	int rc;
 
@@ -791,15 +805,23 @@ int cpu_mmu_map_reserved_page(cpu_page_t * pg)
 		return rc;
 	}
 
+	/* Note: Ideally resereved page mapping should be created
+	 * in each and every l1 table that exist, but that would 
+	 * be uneccesary and redundant. To avoid this we only 
+	 * create mapping in default l1 table and let other VCPUs
+	 * fault on this page and load the page on-demand from
+	 * data abort and prefetch abort handlers.
+	 */
+
 	return VMM_OK;
 }
 
-cpu_l1tbl_t *cpu_mmu_l1tbl_alloc(void)
+struct cpu_l1tbl *cpu_mmu_l1tbl_alloc(void)
 {
 	u32 i, *nl1_tte;
-	cpu_l1tbl_t *nl1 = NULL;
+	struct cpu_l1tbl *nl1 = NULL;
 	struct dlist *le;
-	cpu_l2tbl_t *l2, *nl2;
+	struct cpu_l2tbl *l2, *nl2;
 
 	if (mmuctrl.l1_alloc_count < TTBL_MAX_L1TBL_COUNT) {
 		for (i = 0; i < TTBL_MAX_L1TBL_COUNT; i++) {
@@ -826,7 +848,7 @@ cpu_l1tbl_t *cpu_mmu_l1tbl_alloc(void)
 	nl1->tte_cnt = mmuctrl.defl1.tte_cnt;
 
 	list_for_each(le, &mmuctrl.defl1.l2tbl_list) {
-		l2 = list_entry(le, cpu_l2tbl_t, head);
+		l2 = list_entry(le, struct cpu_l2tbl, head);
 		nl1_tte = (u32 *) (nl1->tbl_va +
 			  ((l2->map_va >> TTBL_L1TBL_TTE_OFFSET_SHIFT) << 2));
 		*nl1_tte = 0x0;
@@ -853,20 +875,20 @@ l1tbl_alloc_fail:
 	if (nl1) {
 		while (!list_empty(&nl1->l2tbl_list)) {
 			le = list_pop(&nl1->l2tbl_list);
-			nl2 = list_entry(le, cpu_l2tbl_t, head);
+			nl2 = list_entry(le, struct cpu_l2tbl, head);
 			cpu_mmu_l2tbl_free(nl2);
 		}
-		mmuctrl.l1_bmap[nl1->l1_num] = 0;
+		mmuctrl.l1_bmap[nl1->num] = 0;
 		mmuctrl.l1_alloc_count--;
 	}
 
 	return NULL;
 }
 
-int cpu_mmu_l1tbl_free(cpu_l1tbl_t * l1)
+int cpu_mmu_l1tbl_free(struct cpu_l1tbl * l1)
 {
 	struct dlist *le;
-	cpu_l2tbl_t *l2;
+	struct cpu_l2tbl *l2;
 
 	if (!l1) {
 		return VMM_EFAIL;
@@ -876,25 +898,25 @@ int cpu_mmu_l1tbl_free(cpu_l1tbl_t * l1)
 	}
 
 	while (!list_empty(&l1->l2tbl_list)) {
-		le = list_pop(&l1->l2tbl_list);
-		l2 = list_entry(le, cpu_l2tbl_t, head);
+		le = list_first(&l1->l2tbl_list);
+		l2 = list_entry(le, struct cpu_l2tbl, head);
 		cpu_mmu_l2tbl_free(l2);
 	}
 
 	list_del(&l1->head);
 
-	mmuctrl.l1_bmap[l1->l1_num] = 0;
+	mmuctrl.l1_bmap[l1->num] = 0;
 	mmuctrl.l1_alloc_count--;
 
 	return VMM_OK;
 }
 
-cpu_l1tbl_t *cpu_mmu_l1tbl_default(void)
+struct cpu_l1tbl *cpu_mmu_l1tbl_default(void)
 {
 	return &mmuctrl.defl1;
 }
 
-cpu_l1tbl_t *cpu_mmu_l1tbl_current(void)
+struct cpu_l1tbl *cpu_mmu_l1tbl_current(void)
 {
 	u32 ttbr0;
 
@@ -907,11 +929,11 @@ u32 cpu_mmu_physical_read32(physical_addr_t pa)
 {
 	u32 ret = 0x0, ite, found;
 	u32 * l1_tte = NULL;
-	cpu_l1tbl_t * l1 = NULL;
+	struct cpu_l1tbl * l1 = NULL;
 	virtual_addr_t va = 0x0;
 	irq_flags_t flags;
 
-	flags = vmm_cpu_irq_save();
+	flags = arch_cpu_irq_save();
 
 	l1 = cpu_mmu_l1tbl_current();
 	if (l1) {
@@ -962,7 +984,7 @@ u32 cpu_mmu_physical_read32(physical_addr_t pa)
 		}
 	}
 
-	vmm_cpu_irq_restore(flags);
+	arch_cpu_irq_restore(flags);
 
 	return ret;
 }
@@ -971,11 +993,11 @@ void cpu_mmu_physical_write32(physical_addr_t pa, u32 val)
 {
 	u32 ite, found;
 	u32 * l1_tte = NULL;
-	cpu_l1tbl_t * l1 = NULL;
+	struct cpu_l1tbl * l1 = NULL;
 	virtual_addr_t va = 0x0;
 	irq_flags_t flags;
 
-	flags = vmm_cpu_irq_save();
+	flags = arch_cpu_irq_save();
 
 	l1 = cpu_mmu_l1tbl_current();
 	if (l1) {
@@ -1026,7 +1048,7 @@ void cpu_mmu_physical_write32(physical_addr_t pa, u32 val)
 		}
 	}
 
-	vmm_cpu_irq_restore(flags);
+	arch_cpu_irq_restore(flags);
 
 	return;
 }
@@ -1047,10 +1069,9 @@ int cpu_mmu_chdacr(u32 new_dacr)
 	return VMM_OK;
 }
 
-int cpu_mmu_chttbr(cpu_l1tbl_t * l1)
+int cpu_mmu_chttbr(struct cpu_l1tbl * l1)
 {
-	u32 sctlr;
-	cpu_l1tbl_t * curr_l1;
+	struct cpu_l1tbl * curr_l1;
 
 	if (!l1) {
 		return VMM_EFAIL;
@@ -1062,43 +1083,34 @@ int cpu_mmu_chttbr(cpu_l1tbl_t * l1)
 		return VMM_OK;
 	}
 
-	sctlr = read_sctlr();
-
-	/* FIXME: Clean & Flush I-Cache if I-Cache enabled */
-	if (sctlr & SCTLR_I_MASK) {
-	}
-
-	/* FIXME: Clean & Flush D-Cache if D-Cache enabled */
-	if (sctlr & SCTLR_C_MASK) {
-	}
-
-	/* Invalidate all TLB enteries */
-	invalid_tlb();
-
 	/* Update TTBR0 to point to new L1 table */
 	write_ttbr0(l1->tbl_pa);
 
+	/* Instruction barrier */
+	isb();
+
+	/* Update Context ID register */
+	write_contextidr((((u32)l1->num) & 0xFF));
+
 	return VMM_OK;
 }
 
-int vmm_cpu_aspace_map(virtual_addr_t va, 
+int arch_cpu_aspace_map(virtual_addr_t va, 
 			virtual_size_t sz, 
 			physical_addr_t pa,
 			u32 mem_flags)
 {
-	cpu_page_t p;
+	struct cpu_page p;
 	vmm_memset(&p, 0, sizeof(p));
 	p.pa = pa;
 	p.va = va;
 	p.sz = sz;
 	p.imp = 0;
 	p.dom = TTBL_L1TBL_TTE_DOM_RESERVED;
-	if (mem_flags & (VMM_MEMORY_READABLE | VMM_MEMORY_WRITEABLE)) {
+	if (mem_flags & VMM_MEMORY_WRITEABLE) {
 		p.ap = TTBL_AP_SRW_U;
 	} else if (mem_flags & VMM_MEMORY_READABLE) {
 		p.ap = TTBL_AP_SR_U;
-	} else if (mem_flags & VMM_MEMORY_WRITEABLE) {
-		p.ap = TTBL_AP_SRW_U;
 	} else {
 		p.ap = TTBL_AP_S_U;
 	}
@@ -1111,11 +1123,11 @@ int vmm_cpu_aspace_map(virtual_addr_t va,
 	return cpu_mmu_map_reserved_page(&p);
 }
 
-int vmm_cpu_aspace_unmap(virtual_addr_t va, 
+int arch_cpu_aspace_unmap(virtual_addr_t va, 
 			 virtual_size_t sz)
 {
 	int rc;
-	cpu_page_t p;
+	struct cpu_page p;
 
 	rc = cpu_mmu_get_reserved_page(va, &p);
 	if (rc) {
@@ -1136,10 +1148,10 @@ int vmm_cpu_aspace_unmap(virtual_addr_t va,
 	return cpu_mmu_unmap_reserved_page(&p);
 }
 
-int vmm_cpu_aspace_va2pa(virtual_addr_t va, physical_addr_t * pa)
+int arch_cpu_aspace_va2pa(virtual_addr_t va, physical_addr_t * pa)
 {
 	int rc = VMM_OK;
-	cpu_page_t p;
+	struct cpu_page p;
 
 	if ((rc = cpu_mmu_get_reserved_page(va, &p))) {
 		return rc;
@@ -1150,7 +1162,7 @@ int vmm_cpu_aspace_va2pa(virtual_addr_t va, physical_addr_t * pa)
 	return VMM_OK;
 }
 
-int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
+int __init arch_cpu_aspace_init(physical_addr_t * resv_pa,
 				virtual_addr_t * resv_va,
 				virtual_size_t * resv_sz)
 {
@@ -1159,7 +1171,7 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 	virtual_addr_t va;
 	virtual_size_t sz;
 	physical_addr_t pa;
-	cpu_page_t respg;
+	struct cpu_page respg;
 
 	/* Reset the memory of MMU control structure */
 	vmm_memset(&mmuctrl, 0, sizeof(mmuctrl));
@@ -1170,11 +1182,12 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 
 	/* Handcraft default translation table */
 	INIT_LIST_HEAD(&mmuctrl.defl1.l2tbl_list);
+	mmuctrl.defl1.num = TTBL_MAX_L1TBL_COUNT;
 	mmuctrl.defl1.tbl_va = (virtual_addr_t)&defl1_mem;
-	mmuctrl.defl1.tbl_pa = vmm_cpu_code_paddr_start() + 
-			       ((virtual_addr_t)&defl1_mem - vmm_cpu_code_vaddr_start());
-	if (vmm_cpu_code_paddr_start() != vmm_cpu_code_vaddr_start()) {
-		val = vmm_cpu_code_paddr_start() >> TTBL_L1TBL_TTE_OFFSET_SHIFT;
+	mmuctrl.defl1.tbl_pa = arch_code_paddr_start() + 
+			       ((virtual_addr_t)&defl1_mem - arch_code_vaddr_start());
+	if (arch_code_paddr_start() != arch_code_vaddr_start()) {
+		val = arch_code_paddr_start() >> TTBL_L1TBL_TTE_OFFSET_SHIFT;
 		val = val << 2;
 		*((u32 *)(mmuctrl.defl1.tbl_va + val)) = 0x0;
 		invalid_tlb();
@@ -1188,11 +1201,12 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 		}
 	}
 	mmuctrl.defl1.l2tbl_cnt = 0;
+	write_contextidr((((u32)mmuctrl.defl1.num) & 0xFF));
 
 	/* Compute additional reserved space required */
-	pa = vmm_cpu_code_paddr_start();
-	va = vmm_cpu_code_vaddr_start();
-	sz = vmm_cpu_code_size();
+	pa = arch_code_paddr_start();
+	va = arch_code_vaddr_start();
+	sz = arch_code_size();
 	if ((va <= *resv_va) && (*resv_va < (va + sz))) {
 		*resv_va = va + sz;
 	} else if ((va <= (*resv_va + *resv_sz)) && 
@@ -1217,14 +1231,14 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 	mmuctrl.l1_bmap = (u8 *)(*resv_va + *resv_sz);
 	*resv_sz += TTBL_MAX_L1TBL_COUNT;
 	*resv_sz = (*resv_sz & 0x3) ? (*resv_sz & ~0x3) + 0x4 : *resv_sz;
-	mmuctrl.l1_array = (cpu_l1tbl_t *)(*resv_va + *resv_sz);
-	*resv_sz += sizeof(cpu_l1tbl_t) * TTBL_MAX_L1TBL_COUNT;
+	mmuctrl.l1_array = (struct cpu_l1tbl *)(*resv_va + *resv_sz);
+	*resv_sz += sizeof(struct cpu_l1tbl) * TTBL_MAX_L1TBL_COUNT;
 	*resv_sz = (*resv_sz & 0x3) ? (*resv_sz & ~0x3) + 0x4 : *resv_sz;
 	mmuctrl.l2_bmap = (u8 *)(*resv_va + *resv_sz);
 	*resv_sz += TTBL_MAX_L2TBL_COUNT;
 	*resv_sz = (*resv_sz & 0x3) ? (*resv_sz & ~0x3) + 0x4 : *resv_sz;
-	mmuctrl.l2_array = (cpu_l2tbl_t *)(*resv_va + *resv_sz);
-	*resv_sz += sizeof(cpu_l2tbl_t) * TTBL_MAX_L2TBL_COUNT;
+	mmuctrl.l2_array = (struct cpu_l2tbl *)(*resv_va + *resv_sz);
+	*resv_sz += sizeof(struct cpu_l2tbl) * TTBL_MAX_L2TBL_COUNT;
 	*resv_sz = (*resv_sz & 0x3) ? (*resv_sz & ~0x3) + 0x4 : *resv_sz;
 	if (*resv_sz & (TTBL_L1TBL_SIZE - 1)) {
 		*resv_sz += TTBL_L1TBL_SIZE - 
@@ -1255,9 +1269,9 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 		respg.ap = TTBL_AP_SRW_U;
 		respg.xn = 0;
 		respg.ng = 0;
-		respg.s = 1;
+		respg.s = 0;
 		respg.c = 1;
-		respg.tex = 0x7;
+		respg.tex = 0;
 		respg.b = 0;
 		if ((rc = cpu_mmu_map_reserved_page(&respg))) {
 			goto mmu_init_error;
@@ -1270,27 +1284,31 @@ int __init vmm_cpu_aspace_init(physical_addr_t * resv_pa,
 	/* Setup up l1 array */
 	vmm_memset(mmuctrl.l1_bmap, 0x0, TTBL_MAX_L1TBL_COUNT);
 	vmm_memset(mmuctrl.l1_array, 0x0, 
-		   sizeof(cpu_l1tbl_t) * TTBL_MAX_L1TBL_COUNT);
+		   sizeof(struct cpu_l1tbl) * TTBL_MAX_L1TBL_COUNT);
 	for (i = 0; i < TTBL_MAX_L1TBL_COUNT; i++) {
 		INIT_LIST_HEAD(&mmuctrl.l1_array[i].head);
-		mmuctrl.l1_array[i].l1_num = i;
+		mmuctrl.l1_array[i].num = i;
 		mmuctrl.l1_array[i].tbl_pa = mmuctrl.l1_base_pa + 
 						i * TTBL_L1TBL_SIZE;
 		mmuctrl.l1_array[i].tbl_va = mmuctrl.l1_base_va + 
 						i * TTBL_L1TBL_SIZE;
+		mmuctrl.l1_array[i].tte_cnt = 0;
+		mmuctrl.l1_array[i].l2tbl_cnt = 0;
+		INIT_LIST_HEAD(&mmuctrl.l1_array[i].l2tbl_list);
 	}
 
 	/* Setup up l2 array */
 	vmm_memset(mmuctrl.l2_bmap, 0x0, TTBL_MAX_L2TBL_COUNT);
 	vmm_memset(mmuctrl.l2_array, 0x0, 
-		   sizeof(cpu_l2tbl_t) * TTBL_MAX_L2TBL_COUNT);
+		   sizeof(struct cpu_l2tbl) * TTBL_MAX_L2TBL_COUNT);
 	for (i = 0; i < TTBL_MAX_L2TBL_COUNT; i++) {
 		INIT_LIST_HEAD(&mmuctrl.l2_array[i].head);
-		mmuctrl.l2_array[i].l2_num = i;
+		mmuctrl.l2_array[i].num = i;
 		mmuctrl.l2_array[i].tbl_pa = mmuctrl.l2_base_pa + 
 						i * TTBL_L2TBL_SIZE;
 		mmuctrl.l2_array[i].tbl_va = mmuctrl.l2_base_va + 
 						i * TTBL_L2TBL_SIZE;
+		mmuctrl.l2_array[i].tte_cnt = 0;
 	}
 
 	return VMM_OK;
