@@ -17,7 +17,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file cpu_emulate_thumb.h
- * @version 1.0
  * @author Anup Patel (anup@brainfault.org)
  * @brief header file to emulate Thumb instructions
  */
@@ -27,8 +26,8 @@
 #include <vmm_types.h>
 
 /** Emulate Priviledged Thumb instructions */
-int cpu_vcpu_emulate_thumb_inst(vmm_vcpu_t *vcpu, 
-				vmm_user_regs_t * regs, 
+int cpu_vcpu_emulate_thumb_inst(struct vmm_vcpu *vcpu, 
+				arch_regs_t * regs, 
 				bool is_hypercall);
 
 #endif
