@@ -17,7 +17,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file cmd_blockdev.c
- * @version 0.01
  * @author Anup Patel (anup@brainfault.org)
  * @brief Implementation of blockdev command
  */
@@ -26,9 +25,9 @@
 #include <vmm_stdio.h>
 #include <vmm_string.h>
 #include <vmm_devtree.h>
-#include <vmm_blockdev.h>
 #include <vmm_modules.h>
 #include <vmm_cmdmgr.h>
+#include <block/vmm_blockdev.h>
 
 #define MODULE_VARID			cmd_blockdev_module
 #define MODULE_NAME			"Command blockdev"
@@ -37,18 +36,18 @@
 #define	MODULE_INIT			cmd_blockdev_init
 #define	MODULE_EXIT			cmd_blockdev_exit
 
-void cmd_blockdev_usage(vmm_chardev_t *cdev)
+void cmd_blockdev_usage(struct vmm_chardev *cdev)
 {
 	vmm_cprintf(cdev, "Usage:\n");
 	vmm_cprintf(cdev, "   blockdev help\n");
 	vmm_cprintf(cdev, "   blockdev list\n");
 }
 
-void cmd_blockdev_list(vmm_chardev_t *cdev)
+void cmd_blockdev_list(struct vmm_chardev *cdev)
 {
 	int num, count;
 	char path[1024];
-	vmm_blockdev_t *bdev;
+	struct vmm_blockdev *bdev;
 	count = vmm_blockdev_count();
 	for (num = 0; num < count; num++) {
 		bdev = vmm_blockdev_get(num);
@@ -61,7 +60,7 @@ void cmd_blockdev_list(vmm_chardev_t *cdev)
 	}
 }
 
-int cmd_blockdev_exec(vmm_chardev_t *cdev, int argc, char **argv)
+int cmd_blockdev_exec(struct vmm_chardev *cdev, int argc, char **argv)
 {
 	if (argc == 2) {
 		if (vmm_strcmp(argv[1], "help") == 0) {
@@ -79,7 +78,7 @@ int cmd_blockdev_exec(vmm_chardev_t *cdev, int argc, char **argv)
 	return VMM_OK;
 }
 
-static vmm_cmd_t cmd_blockdev = {
+static struct vmm_cmd cmd_blockdev = {
 	.name = "blockdev",
 	.desc = "block device commands",
 	.usage = cmd_blockdev_usage,
