@@ -17,7 +17,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file vmm_chardev.c
- * @version 1.0
  * @author Anup Patel (anup@brainfault.org)
  * @brief Character Device framework source
  */
@@ -28,77 +27,71 @@
 #include <vmm_heap.h>
 #include <vmm_chardev.h>
 
-int vmm_chardev_doioctl(vmm_chardev_t * cdev,
+int vmm_chardev_doioctl(struct vmm_chardev * cdev,
 			int cmd, void *buf, size_t len, bool block)
 {
-	if (!cdev) {
+	if (cdev && cdev->ioctl) {
+		return cdev->ioctl(cdev, cmd, buf, len, block);
+	} else {
 		return VMM_EFAIL;
 	}
-	if (!(cdev->ioctl)) {
-		return VMM_EFAIL;
-	}
-
-	return cdev->ioctl(cdev, cmd, buf, len, block);
 }
 
-u32 vmm_chardev_doread(vmm_chardev_t * cdev,
+u32 vmm_chardev_doread(struct vmm_chardev * cdev,
 		       u8 *dest, size_t offset, size_t len, bool block)
 {
-	if (!cdev) {
+	if (cdev && cdev->read) {
+		return cdev->read(cdev, dest, offset, len, block);
+	} else {
 		return 0;
 	}
-	if (!(cdev->read)) {
-		return 0;
-	}
-
-	return cdev->read(cdev, dest, offset, len, block);
 }
 
-u32 vmm_chardev_dowrite(vmm_chardev_t * cdev,
+u32 vmm_chardev_dowrite(struct vmm_chardev * cdev,
 			u8 *src, size_t offset, size_t len, bool block)
 {
-	if (!cdev) {
+	if (cdev && cdev->write) {
+		return cdev->write(cdev, src, offset, len, block);
+	} else {
 		return 0;
 	}
-	if (!(cdev->write)) {
-		return 0;
-	}
-
-	return cdev->write(cdev, src, offset, len, block);
 }
 
-int vmm_chardev_register(vmm_chardev_t * cdev)
+int vmm_chardev_register(struct vmm_chardev * cdev)
 {
-	vmm_classdev_t *cd;
+	int rc;
+	struct vmm_classdev *cd;
 
-	if (cdev == NULL) {
-		return VMM_EFAIL;
-	}
-	if (cdev->read == NULL || cdev->write == NULL) {
+	if (!(cdev && cdev->read && cdev->write)) {
 		return VMM_EFAIL;
 	}
 
-	cd = vmm_malloc(sizeof(vmm_classdev_t));
+	cd = vmm_malloc(sizeof(struct vmm_classdev));
 	if (!cd) {
 		return VMM_EFAIL;
 	}
 
+	vmm_memset(cd, 0, sizeof(struct vmm_classdev));
+
 	INIT_LIST_HEAD(&cd->head);
 	vmm_strcpy(cd->name, cdev->name);
 	cd->dev = cdev->dev;
 	cd->priv = cdev;
 
-	vmm_devdrv_register_classdev(VMM_CHARDEV_CLASS_NAME, cd);
+	rc = vmm_devdrv_register_classdev(VMM_CHARDEV_CLASS_NAME, cd);
+	if (rc != VMM_OK) {
+		vmm_free(cd);
+	}
 
-	return VMM_OK;
+	return rc;
 }
 
-int vmm_chardev_unregister(vmm_chardev_t * cdev)
+int vmm_chardev_unregister(struct vmm_chardev * cdev)
 {
 	int rc;
-	vmm_classdev_t *cd;
+	struct vmm_classdev *cd;
 
-	if (cdev == NULL) {
+	if (!cdev) {
 		return VMM_EFAIL;
 	}
 
@@ -108,20 +101,18 @@ int vmm_chardev_unregister(vmm_chardev_t * cdev)
 	}
 
 	rc = vmm_devdrv_unregister_classdev(VMM_CHARDEV_CLASS_NAME, cd);
-
-	if (!rc) {
+	if (rc == VMM_OK) {
 		vmm_free(cd);
 	}
 
 	return rc;
 }
 
-vmm_chardev_t *vmm_chardev_find(const char *name)
+struct vmm_chardev *vmm_chardev_find(const char *name)
 {
-	vmm_classdev_t *cd;
+	struct vmm_classdev *cd;
 
 	cd = vmm_devdrv_find_classdev(VMM_CHARDEV_CLASS_NAME, name);
-
 	if (!cd) {
 		return NULL;
 	}
@@ -129,12 +120,11 @@ vmm_chardev_t *vmm_chardev_find(const char *name)
 	return cd->priv;
 }
 
-vmm_chardev_t *vmm_chardev_get(int num)
+struct vmm_chardev *vmm_chardev_get(int num)
 {
-	vmm_classdev_t *cd;
+	struct vmm_classdev *cd;
 
 	cd = vmm_devdrv_classdev(VMM_CHARDEV_CLASS_NAME, num);
-
 	if (!cd) {
 		return NULL;
 	}
@@ -150,22 +140,23 @@ u32 vmm_chardev_count(void)
 int __init vmm_chardev_init(void)
 {
 	int rc;
-	vmm_class_t *c;
+	struct vmm_class *c;
 
-	c = vmm_malloc(sizeof(vmm_class_t));
+	c = vmm_malloc(sizeof(struct vmm_class));
 	if (!c) {
 		return VMM_EFAIL;
 	}
 
+	vmm_memset(c, 0, sizeof(struct vmm_class));
+
 	INIT_LIST_HEAD(&c->head);
 	vmm_strcpy(c->name, VMM_CHARDEV_CLASS_NAME);
 	INIT_LIST_HEAD(&c->classdev_list);
 
 	rc = vmm_devdrv_register_class(c);
-	if (rc) {
+	if (rc != VMM_OK) {
 		vmm_free(c);
-		return rc;
 	}
 
-	return VMM_OK;
+	return rc;
 }
